@@ -26,7 +26,7 @@ import json
 from requests.sessions import Session
 from requests.utils import get_environ_proxies
 
-import botocore.auth
+from botocore.auth import AUTH_TYPE_MAPS, UnknownSignatureVersionError
 import botocore.response
 import botocore.exceptions
 from botocore.awsrequest import AWSRequest
@@ -70,7 +70,8 @@ class Endpoint(object):
 
     def prepare_request(self, request):
         logger.debug('prepare_request')
-        self.auth.add_auth(request=request)
+        if self.auth is not None:
+            self.auth.add_auth(request=request)
         prepared_request = request.prepare()
         return prepared_request
 
@@ -100,7 +101,8 @@ class QueryEndpoint(Endpoint):
                              data=params, headers={'User-Agent': user_agent})
         prepared_request = self.prepare_request(request)
         http_response = self._send_request(prepared_request, operation)
-        return botocore.response.get_response(operation, http_response)
+        return botocore.response.get_response(self.session, operation,
+                                              http_response)
 
 
 class JSONEndpoint(Endpoint):
@@ -135,7 +137,8 @@ class JSONEndpoint(Endpoint):
                                       'Content-Encoding': content_encoding})
         prepared_request = self.prepare_request(request)
         http_response = self._send_request(prepared_request, operation)
-        return botocore.response.get_response(operation, http_response)
+        return botocore.response.get_response(self.session, operation,
+                                              http_response)
 
 
 class RestEndpoint(Endpoint):
@@ -197,7 +200,8 @@ class RestEndpoint(Endpoint):
                                  data=params['payload'])
         prepared_request = self.prepare_request(request)
         http_response = self._send_request(prepared_request, operation)
-        return botocore.response.get_response(operation, http_response)
+        return botocore.response.get_response(self.session, operation,
+                                              http_response)
 
 
 def _get_proxies(url):
@@ -207,24 +211,35 @@ def _get_proxies(url):
 
 
 def get_endpoint(service, region_name, endpoint_url):
-    service_to_endpoint = {
-        'query': QueryEndpoint,
-        'json': JSONEndpoint,
-        'rest-xml': RestEndpoint,
-        'rest-json': RestEndpoint,
-    }
     service_type = service.type
-    if service_type not in service_to_endpoint:
+    cls = SERVICE_TO_ENDPOINT.get(service_type)
+    if cls is None:
         raise botocore.exceptions.UnknownServiceStyle(
             service_style=service.type)
-    cls = service_to_endpoint.get(service_type)
-    if cls is None:
-        raise NotImplementedError("%s service type is not yet implemented" %
-                                  service_type)
     service_name = getattr(service, 'signing_name', service.endpoint_prefix)
-    auth = botocore.auth.get_auth(service.signature_version,
-                                  credentials=service.session.get_credentials(),
-                                  service_name=service_name,
-                                  region_name=region_name)
+    auth = None
+    if hasattr(service, 'signature_version'):
+        auth = _get_auth(service.signature_version,
+            credentials=service.session.get_credentials(),
+            service_name=service_name,
+            region_name=region_name)
     proxies = _get_proxies(endpoint_url)
     return cls(service, region_name, endpoint_url, auth=auth, proxies=proxies)
+
+
+def _get_auth(signature_version, credentials, service_name, region_name):
+    cls = AUTH_TYPE_MAPS.get(signature_version)
+    if cls is None:
+        raise UnknownSignatureVersionError(signature_version=signature_version)
+    else:
+        return cls(credentials=credentials,
+                   service_name=service_name,
+                   region_name=region_name)
+
+
+SERVICE_TO_ENDPOINT = {
+    'query': QueryEndpoint,
+    'json': JSONEndpoint,
+    'rest-xml': RestEndpoint,
+    'rest-json': RestEndpoint,
+}
