@@ -23,6 +23,7 @@
 import sys
 import xml.etree.cElementTree
 from botocore import ScalarTypes
+from .hooks import first_non_none_response
 import json
 import logging
 
@@ -31,11 +32,12 @@ logger = logging.getLogger(__name__)
 
 class Response(object):
 
-    def __init__(self, operation):
+    def __init__(self, session, operation):
+        self.session = session
         self.operation = operation
         self.value = None
 
-    def parse(self, s):
+    def parse(self, s, encoding):
         pass
 
     def get_value(self):
@@ -44,8 +46,8 @@ class Response(object):
 
 class XmlResponse(Response):
 
-    def __init__(self, operation):
-        Response.__init__(self, operation)
+    def __init__(self, session, operation):
+        Response.__init__(self, session, operation)
         self.tree = None
         self.element_map = {}
         self.value = {}
@@ -61,8 +63,12 @@ class XmlResponse(Response):
             elem_tag = elem.tag
         return elem_tag
 
-    def parse(self, s):
-        self.tree = xml.etree.cElementTree.fromstring(s)
+    def parse(self, s, encoding):
+        parser = xml.etree.cElementTree.XMLParser(
+            target=xml.etree.cElementTree.TreeBuilder(),
+            encoding=encoding)
+        parser.feed(s)
+        self.tree = parser.close()
         if self.operation.output:
             self.build_element_map(self.operation.output, 'root')
         self.start(self.tree)
@@ -76,10 +82,37 @@ class XmlResponse(Response):
             request_id = rmd_elem.find(self.clark_notation('RequestId'))
         else:
             request_id = self.tree.find(self.clark_notation('requestId'))
+            if request_id is None:
+                request_id = self.tree.find(self.clark_notation('RequestId'))
+            if request_id is None:
+                request_id = self.tree.find('RequestID')
         if request_id is not None:
             request_id.tail = True
             rmd['RequestId'] = request_id.text.strip()
 
+    def _get_error_data(self, error_elem):
+        data = {}
+        for elem in error_elem:
+            elem.tail = True
+            data[self.get_element_base_tag(elem)] = elem.text
+        return data
+
+    def get_response_errors(self):
+        errors = None
+        error_elems = self.tree.find('Errors')
+        if error_elems is not None:
+            error_elems.tail = True
+            errors = [self._get_error_data(e) for e in error_elems]
+        else:
+            error_elems = self.tree.find(self.clark_notation('Error'))
+            if error_elems is not None:
+                error_elems.tail = True
+                errors = [self._get_error_data(error_elems)]
+            elif self.tree.tag == 'Error':
+                errors = [self._get_error_data(self.tree)]
+        if errors:
+            self.value['Errors'] = errors
+
     def build_element_map(self, defn, keyname):
         xmlname = defn.get('xmlname', keyname)
         if not xmlname:
@@ -173,9 +206,8 @@ class XmlResponse(Response):
             xmlname = member_shape.get('xmlname', member_name)
             child = self.find(elem, xmlname)
             if child is not None:
-                new_data[member_name] = self.handle_elem(child, member_shape)
-            else:
-                logger.debug('unable to find struct member: %s' % xmlname)
+                new_data[member_name] = self.handle_elem(member_name, child,
+                                                         member_shape)
         return new_data
 
     def _handle_list(self, elem, shape):
@@ -188,7 +220,7 @@ class XmlResponse(Response):
                 children = self.findall(parent, tagname)
         if not children:
             children = []
-        return [self.handle_elem(child, shape['members'])
+        return [self.handle_elem(None, child, shape['members'])
                 for child in children]
 
     def _handle_map(self, elem, shape):
@@ -209,15 +241,31 @@ class XmlResponse(Response):
             value = self.find(member, value_xmlname)
             cn = self.clark_notation(value_xmlname)
             value = member.find(cn)
-            data[self.handle_elem(key, keyshape)] = self.handle_elem(value,
-                                                                     valueshape)
+            key_name = self.handle_elem(None, key, keyshape)
+            data[key_name] = self.handle_elem(key_name, value, valueshape)
         return data
 
-    def handle_elem(self, elem, shape):
+    def emit_event(self, tag, shape, value):
+        if 'shape_name' in shape:
+            event = self.session.create_event('after-parsed',
+                                              self.operation.service.endpoint_prefix,
+                                              self.operation.name,
+                                              shape['shape_name'], tag)
+            rv = first_non_none_response(self.session.emit(event,
+                                                           shape=shape,
+                                                           value=value),
+                                         None)
+            if rv:
+                value = rv
+        return value
+
+    def handle_elem(self, key, elem, shape):
         handler_name = '_handle_%s' % shape['type']
         elem.tail = True
         if hasattr(self, handler_name):
-            return getattr(self, handler_name)(elem, shape)
+            value = getattr(self, handler_name)(elem, shape)
+            value = self.emit_event(key, shape, value)
+            return value
         else:
             logger.debug('Unhandled type: %s' % shape['type'])
 
@@ -242,6 +290,7 @@ class XmlResponse(Response):
         return shape
 
     def start(self, elem):
+        logger.debug('start')
         self.value = {}
         if self.operation.output:
             for member_name in self.operation.output['members']:
@@ -251,30 +300,43 @@ class XmlResponse(Response):
                 if child is None and member['type'] not in ScalarTypes:
                     child = elem
                 if child is not None:
-                    self.value[member_name] = self.handle_elem(child, member)
+                    self.value[member_name] = self.handle_elem(member_name,
+                                                               child, member)
         self.get_response_metadata()
+        self.get_response_errors()
         for child in self.tree:
             if child.tail is not True:
                 child_tag = self.get_element_base_tag(child)
                 if child_tag not in self.element_map:
                     if not child_tag.startswith(self.operation.name):
                         shape = self.fake_shape(child)
-                        self.value[child_tag] = self.handle_elem(child, shape)
+                        self.value[child_tag] = self.handle_elem(child_tag,
+                                                                 child, shape)
 
 
 class JSONResponse(Response):
 
-    def parse(self, s):
+    def parse(self, s, encoding):
         try:
-            self.value = json.loads(s)
-        except:
-            logger.debug('Error loading JSON response body')
+            self.value = json.loads(s, encoding=encoding)
+            self.get_response_errors()
+        except Exception as err:
+            logger.debug('Error loading JSON response body, %r', err)
+
+    def get_response_errors(self):
+        if '__type' in self.value:
+            error = {'Type': self.value['__type']}
+            del self.value['__type']
+            if 'message' in self.value:
+                error['Message'] = self.value['message']
+                del self.value['message']
+            self.value['Errors'] = [error]
 
 
 class StreamingResponse(Response):
 
-    def __init__(self, operation):
-        Response.__init__(self, operation)
+    def __init__(self, session, operation):
+        Response.__init__(self, session, operation)
         self.value = {}
 
     def parse(self, headers, stream):
@@ -290,7 +352,7 @@ class StreamingResponse(Response):
                         self.value[member_name] = stream
 
 
-def get_response(operation, http_response):
+def get_response(session, operation, http_response):
     encoding = 'utf-8'
     if http_response.encoding:
         encoding = http_response.encoding
@@ -299,21 +361,21 @@ def get_response(operation, http_response):
         content_type = content_type.split(';')[0]
     logger.debug('content-type=%s' % content_type)
     if operation.is_streaming():
-        streaming_response = StreamingResponse(operation)
+        streaming_response = StreamingResponse(session, operation)
         streaming_response.parse(http_response.headers, http_response.raw)
         return (http_response, streaming_response.get_value())
-    body = http_response.text.encode(encoding.lower())
+    body = http_response.text
     logger.debug("Response Body: %s", body)
     if not body:
         return (http_response, body)
     if content_type in ('application/x-amz-json-1.0',
                         'application/x-amz-json-1.1', 'application/json'):
-        json_response = JSONResponse(operation)
-        json_response.parse(body)
+        json_response = JSONResponse(session, operation)
+        json_response.parse(body, encoding)
         return (http_response, json_response.get_value())
     # We are defaulting to an XML response handler because many query
     # services send XML error responses but do not include a Content-Type
     # header.
-    xml_response = XmlResponse(operation)
-    xml_response.parse(body)
+    xml_response = XmlResponse(session, operation)
+    xml_response.parse(body, encoding)
     return (http_response, xml_response.get_value())
